@@ -1,37 +1,33 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; income-statement.scm: income statement (a.k.a. Profit & Loss)
-;; 
+;;
 ;; By David Montenegro <sunrise2000@comcast.net>
 ;;  2004.07.13 - 2004.07.14
 ;;
 ;;  * BUGS:
-;;    
+;;
 ;;    This code makes the assumption that you want your income
 ;;    statement to no more than daily resolution.
-;;    
-;;    The Company Name field does not currently default to the name
-;;    in (gnc-get-current-book).
-;;    
+;;
 ;;    Line & column alignments may still not conform with
 ;;    textbook accounting practice (they're close though!).
-;;    The 'canonically-tabbed option is currently broken.
-;;    
+;;
 ;;    Progress bar functionality is currently mostly broken.
-;;    
+;;
 ;;    The variables in this code could use more consistent naming.
-;;    
+;;
 ;;    See also all the "FIXME"s in the code.
-;;    
-;; This program is free software; you can redistribute it and/or    
-;; modify it under the terms of the GNU General Public License as   
-;; published by the Free Software Foundation; either version 2 of   
-;; the License, or (at your option) any later version.              
-;;                                                                  
-;; This program is distributed in the hope that it will be useful,  
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
-;; GNU General Public License for more details.                     
-;;                                                                  
+;;
+;; This program is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2 of
+;; the License, or (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
 ;; You should have received a copy of the GNU General Public License
 ;; along with this program; if not, contact:
 ;;
@@ -53,9 +49,6 @@
 (define optname-report-title (N_ "Report Title"))
 (define opthelp-report-title (N_ "Title for this report."))
 
-(define optname-party-name (N_ "Company name"))
-(define opthelp-party-name (N_ "Name of company/individual."))
-
 (define optname-start-date (N_ "Start Date"))
 (define optname-end-date (N_ "End Date"))
 
@@ -137,74 +130,58 @@
 
 ;; options generator
 (define (income-statement-options-generator-internal reportname)
-  (let* ((options (gnc:new-options))
-         (book (gnc-get-current-book)) ; XXX Find a way to get the book that opened the report
-         (add-option 
-          (lambda (new-option)
-            (gnc:register-option options new-option))))
-    
-    (add-option
-      (gnc:make-string-option
+  (let* ((options (gnc-new-optiondb)))
+
+    (gnc-register-string-option options
       gnc:pagename-general optname-report-title
-      "a" opthelp-report-title (G_ reportname)))
-    (add-option
-      (gnc:make-string-option
-      gnc:pagename-general optname-party-name
-      "b" opthelp-party-name (or (gnc:company-info book gnc:*company-name*) "")))
-    
+      "a" opthelp-report-title (G_ reportname))
+
     ;; period over which to report income
     (gnc:options-add-date-interval!
-     options gnc:pagename-general 
+     options gnc:pagename-general
      optname-start-date optname-end-date "c")
-    
+
     ;; accounts to work on
-    (add-option
-     (gnc:make-account-list-option
+    (gnc-register-account-list-option options
       gnc:pagename-accounts optname-accounts
       "a"
       opthelp-accounts
-      (lambda ()
-	(gnc:filter-accountlist-type
-	 ;; select, by default, only income and expense accounts
-	 (list ACCT-TYPE-INCOME ACCT-TYPE-EXPENSE)
-	 (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
-      #f #t))
+        (gnc:filter-accountlist-type
+         ;; select, by default, only income and expense accounts
+         (list ACCT-TYPE-INCOME ACCT-TYPE-EXPENSE)
+         (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
+
     (gnc:options-add-account-levels!
      options gnc:pagename-accounts optname-depth-limit
      "b" opthelp-depth-limit 3)
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-accounts optname-bottom-behavior
-      "c" opthelp-bottom-behavior #f))
-    
+      "c" opthelp-bottom-behavior #f)
+
     ;; all about currencies
     (gnc:options-add-currency!
      options pagename-commodities
      optname-report-commodity "a")
-    
-    (gnc:options-add-price-source! 
+
+    (gnc:options-add-price-source!
      options pagename-commodities
      optname-price-source "b" 'pricedb-nearest)
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
-      pagename-commodities optname-show-foreign 
-      "c" opthelp-show-foreign #t))
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
+
+    (gnc-register-simple-boolean-option options
+      pagename-commodities optname-show-foreign
+      "c" opthelp-show-foreign #t)
+
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-rates
-      "d" opthelp-show-rates #f))
-    
+      "d" opthelp-show-rates #f)
+
     ;; what to show for zero-balance accounts
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-zb-accts
-      "a" opthelp-show-zb-accts #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "a" opthelp-show-zb-accts #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-omit-zb-bals
-      "b" opthelp-omit-zb-bals #f))
+      "b" opthelp-omit-zb-bals #f)
     ;; what to show for non-leaf accounts
     (gnc:options-add-subtotal-view!
      options gnc:pagename-display
@@ -212,72 +189,59 @@
      "c")
 
     ;; some detailed formatting options
-    (add-option 
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-account-links
-      "e" opthelp-account-links #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "e" opthelp-account-links #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-use-rules
-      "f" opthelp-use-rules #f))
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "f" opthelp-use-rules #f)
+
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-label-revenue
-      "g" opthelp-label-revenue #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "g" opthelp-label-revenue #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-total-revenue
-      "h" opthelp-total-revenue #t))
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "h" opthelp-total-revenue #t)
+
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-label-trading
-      "h1" opthelp-label-trading #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "h1" opthelp-label-trading #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-total-trading
-      "h2" opthelp-total-trading #t))
-    
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "h2" opthelp-total-trading #t)
+
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-label-expense
-      "i" opthelp-label-expense #t))
-    (add-option 
-     (gnc:make-simple-boolean-option
+      "i" opthelp-label-expense #t)
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-total-expense
-      "j" opthelp-total-expense #t))
+      "j" opthelp-total-expense #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-two-column
-      "k" opthelp-two-column #f))
+      "k" opthelp-two-column #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-standard-order
-      "l" opthelp-standard-order #t))
-    
+      "l" opthelp-standard-order #t)
+
     ;; closing entry match criteria
-    ;; 
+    ;;
     ;; N.B.: transactions really should have a field where we can put
     ;; transaction types like "Adjusting/Closing/Correcting Entries"
-    (add-option
-      (gnc:make-string-option
+    (gnc-register-string-option options
       pagename-entries optname-closing-pattern
-      "a" opthelp-closing-pattern (G_ "Closing Entries")))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "a" opthelp-closing-pattern (G_ "Closing Entries"))
+    (gnc-register-simple-boolean-option options
       pagename-entries optname-closing-casing
-      "b" opthelp-closing-casing #f))
-    (add-option
-     (gnc:make-simple-boolean-option
+      "b" opthelp-closing-casing #f)
+    (gnc-register-simple-boolean-option options
       pagename-entries optname-closing-regexp
-      "c" opthelp-closing-regexp #f))
-    
+      "c" opthelp-closing-regexp #f)
+
     ;; Set the accounts page as default option tab
     (gnc:options-set-default-section options gnc:pagename-accounts)
-    
+
     options))
 
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -287,33 +251,32 @@
 
 (define (income-statement-renderer-internal report-obj reportname)
   (define (get-option pagename optname)
-    (gnc:option-value
-     (gnc:lookup-option 
-      (gnc:report-options report-obj) pagename optname)))
+    (gnc-optiondb-lookup-value
+      (gnc:report-options report-obj) pagename optname))
 
   (gnc:report-starting reportname)
-  
+
   ;; get all option's values
   (let* (
-	 (report-title (get-option gnc:pagename-general optname-report-title))
-	 (company-name (get-option gnc:pagename-general optname-party-name))
+         (report-title (get-option gnc:pagename-general optname-report-title))
+         (company-name (or (gnc:company-info (gnc-get-current-book) gnc:*company-name*) ""))
          (start-date-printable (gnc:date-option-absolute-time
-				(get-option gnc:pagename-general
-					    optname-start-date)))
+                                (get-option gnc:pagename-general
+                                            optname-start-date)))
          (start-date (gnc:time64-start-day-time
-			 (gnc:date-option-absolute-time
-			  (get-option gnc:pagename-general
-				      optname-start-date))))
+                         (gnc:date-option-absolute-time
+                          (get-option gnc:pagename-general
+                                      optname-start-date))))
          (end-date (gnc:time64-end-day-time
-		       (gnc:date-option-absolute-time
-			(get-option gnc:pagename-general
-				    optname-end-date))))
+                       (gnc:date-option-absolute-time
+                        (get-option gnc:pagename-general
+                                    optname-end-date))))
          (accounts (get-option gnc:pagename-accounts
-                               optname-accounts))	 
-	 (depth-limit (get-option gnc:pagename-accounts 
-				  optname-depth-limit))
-	 (bottom-behavior (get-option gnc:pagename-accounts 
-				  optname-bottom-behavior))
+                               optname-accounts))
+         (depth-limit (get-option gnc:pagename-accounts
+                                  optname-depth-limit))
+         (bottom-behavior (get-option gnc:pagename-accounts
+                                  optname-bottom-behavior))
          (report-commodity (get-option pagename-commodities
                                       optname-report-commodity))
          (price-source (get-option pagename-commodities
@@ -325,64 +288,64 @@
          (parent-balance-mode (get-option gnc:pagename-display
                                            optname-parent-balance-mode))
          (parent-total-mode
-	  (assq-ref '((t . #t) (f . #f) (canonically-tabbed . canonically-tabbed))
-		    (get-option gnc:pagename-display
-				optname-parent-total-mode)))
+          (assq-ref '((t . #t) (f . #f))
+                    (get-option gnc:pagename-display
+                                optname-parent-total-mode)))
          (show-zb-accts? (get-option gnc:pagename-display
-				     optname-show-zb-accts))
+                                     optname-show-zb-accts))
          (omit-zb-bals? (get-option gnc:pagename-display
-				    optname-omit-zb-bals))
+                                    optname-omit-zb-bals))
          (label-revenue? (get-option gnc:pagename-display
-				    optname-label-revenue))
+                                    optname-label-revenue))
          (total-revenue? (get-option gnc:pagename-display
-				    optname-total-revenue))
+                                    optname-total-revenue))
          (label-trading? (get-option gnc:pagename-display
-				    optname-label-trading))
+                                    optname-label-trading))
          (total-trading? (get-option gnc:pagename-display
-				    optname-total-trading))
+                                    optname-total-trading))
          (label-expense? (get-option gnc:pagename-display
-				    optname-label-expense))
+                                    optname-label-expense))
          (total-expense? (get-option gnc:pagename-display
-				    optname-total-expense))
+                                    optname-total-expense))
          (use-links? (get-option gnc:pagename-display
-				     optname-account-links))
+                                     optname-account-links))
          (use-rules? (get-option gnc:pagename-display
-				    optname-use-rules))
-	 (closing-str (get-option pagename-entries
-				  optname-closing-pattern))
-	 (closing-cased (get-option pagename-entries
-				    optname-closing-casing))
-	 (closing-regexp (get-option pagename-entries
-				     optname-closing-regexp))
-	 (two-column? (get-option gnc:pagename-display
-				  optname-two-column))
-	 (standard-order? (get-option gnc:pagename-display
-				      optname-standard-order))
-	 (closing-pattern
-	  (list (list 'str closing-str)
-		(list 'cased closing-cased)
-		(list 'regexp closing-regexp)
-		(list 'closing #t)))
+                                    optname-use-rules))
+         (closing-str (get-option pagename-entries
+                                  optname-closing-pattern))
+         (closing-cased (get-option pagename-entries
+                                    optname-closing-casing))
+         (closing-regexp (get-option pagename-entries
+                                     optname-closing-regexp))
+         (two-column? (get-option gnc:pagename-display
+                                  optname-two-column))
+         (standard-order? (get-option gnc:pagename-display
+                                      optname-standard-order))
+         (closing-pattern
+          (list (list 'str closing-str)
+                (list 'cased closing-cased)
+                (list 'regexp closing-regexp)
+                (list 'closing #t)))
 
          ;; decompose the account list
          (split-up-accounts (gnc:decompose-accountlist accounts))
-	 (revenue-accounts (assoc-ref split-up-accounts ACCT-TYPE-INCOME))
-	 (trading-accounts (assoc-ref split-up-accounts ACCT-TYPE-TRADING))
-	 (expense-accounts (assoc-ref split-up-accounts ACCT-TYPE-EXPENSE))
-	 
+         (revenue-accounts (assoc-ref split-up-accounts ACCT-TYPE-INCOME))
+         (trading-accounts (assoc-ref split-up-accounts ACCT-TYPE-TRADING))
+         (expense-accounts (assoc-ref split-up-accounts ACCT-TYPE-EXPENSE))
+
          (doc (gnc:make-html-document))
-	 ;; this can occasionally put extra (blank) columns in our
-	 ;; table (when there is one account at the maximum depth and
-	 ;; it has at least one of its ancestors deselected), but this
-	 ;; is the only simple way to ensure that both tables
-	 ;; (revenue, expense) have the same width.
+         ;; this can occasionally put extra (blank) columns in our
+         ;; table (when there is one account at the maximum depth and
+         ;; it has at least one of its ancestors deselected), but this
+         ;; is the only simple way to ensure that both tables
+         ;; (revenue, expense) have the same width.
          (tree-depth (if (equal? depth-limit 'all)
-                         (gnc:get-current-account-tree-depth) 
-			 depth-limit))
+                         (gnc:get-current-account-tree-depth)
+                         depth-limit))
          ;; exchange rates calculation parameters
-	 (exchange-fn
-	  (gnc:case-exchange-fn price-source report-commodity end-date))
-	 )
+         (exchange-fn
+          (gnc:case-exchange-fn price-source report-commodity end-date))
+         (price-fn (gnc:case-price-fn price-source report-commodity end-date)))
 
     ;; Wrapper to call gnc:html-table-add-labeled-amount-line!
     ;; with the proper arguments.
@@ -404,10 +367,11 @@
       (gnc:html-table-append-ruler! table (* 2 tree-depth)))
 
     (gnc:html-document-set-title!
-     doc (format #f (string-append "~a ~a " (G_ "For Period Covering ~a to ~a"))
-                  company-name report-title
-                  (qof-print-date start-date-printable)
-                  (qof-print-date end-date)))
+     doc (gnc:format (G_ "${company-name} ${report-title} For Period Covering ${start} to ${end}")
+                     'company-name company-name
+                     'report-title report-title
+                     'start (qof-print-date start-date-printable)
+                     'end (qof-print-date end-date)))
 
     (if (null? accounts)
 
@@ -442,10 +406,7 @@
                                 trading-total
                                 (gnc:collector- expense-total)))
 
-               (inc-table (gnc:make-html-table))
-               (exp-table (gnc:make-html-table))
-               (tra-table (gnc:make-html-table))
-
+               (build-table (gnc:make-html-table))
                (table-env
                 (list
                  (list 'start-date start-date)
@@ -495,74 +456,93 @@
                label                0  1 "text-cell"
                bal          (+ col 1)  1 "number-cell")))
 
-          (let ((space (make-list tree-depth (gnc:make-html-table-cell/min-width 60))))
-            (gnc:html-table-append-row! inc-table space)
-            (gnc:html-table-append-row! exp-table space)
-            (gnc:html-table-append-row! tra-table space))
-          (gnc:report-percent-done 80)
+          (define (add-revenue-table table)
+            (when label-revenue?
+              (add-subtotal-line table (G_ "Revenues") #f #f))
+            (gnc:html-table-add-account-balances table revenue-table params)
+            (when total-revenue?
+              (add-subtotal-line table (G_ "Total Revenue") #f revenue-total))
+            table)
 
-          (when label-revenue?
-            (add-subtotal-line inc-table (G_ "Revenues") #f #f))
-          (gnc:html-table-add-account-balances inc-table revenue-table params)
-          (when total-revenue?
-            (add-subtotal-line inc-table (G_ "Total Revenue") #f revenue-total))
-          (gnc:report-percent-done 85)
+          (define (add-expense-table table)
+            (when label-expense?
+              (add-subtotal-line table (G_ "Expenses") #f #f))
+            (gnc:html-table-add-account-balances table expense-table params)
+            (when total-expense?
+              (add-subtotal-line table (G_ "Total Expenses") #f expense-total))
+            table)
 
-          (when label-expense?
-            (add-subtotal-line exp-table (G_ "Expenses") #f #f))
-          (gnc:html-table-add-account-balances exp-table expense-table params)
-          (when total-expense?
-            (add-subtotal-line exp-table (G_ "Total Expenses") #f expense-total))
+          (define (add-trading-table table)
+            (when label-trading?
+              (add-subtotal-line table (G_ "Trading") #f #f))
+            (gnc:html-table-add-account-balances table trading-table params)
+            (when total-trading?
+              (add-subtotal-line table (G_ "Total Trading") #f trading-total))
+            table)
 
-          (when label-trading?
-            (add-subtotal-line tra-table (G_ "Trading") #f #f))
-          (gnc:html-table-add-account-balances tra-table trading-table params)
-          (when total-trading?
-            (add-subtotal-line tra-table (G_ "Total Trading") #f trading-total))
+          (cond
+           (two-column?
+            (let* ((exp-table (add-expense-table (gnc:make-html-table)))
+                   (inc-table (add-revenue-table (gnc:make-html-table)))
+                   (tra-table (add-trading-table (gnc:make-html-table)))
+                   (inc-cell (gnc:make-html-table-cell inc-table))
+                   (tra-cell (if (null? trading-accounts)
+                                 (gnc:html-make-empty-cell)
+                                 (gnc:make-html-table-cell tra-table)))
+                   (exp-cell (gnc:make-html-table-cell exp-table)))
+              (define (add-cells . lst) (gnc:html-table-append-row! build-table lst))
+              (add-rule (if standard-order? exp-table inc-table))
+              (add-report-line
+               (if standard-order? exp-table inc-table)
+               (string-append (G_ "Net income") period-for)
+               (string-append (G_ "Net loss") period-for)
+               net-income (* 2 (1- tree-depth)) exchange-fn #f #f)
+              (if standard-order?
+                  (add-cells inc-cell tra-cell exp-cell)
+                  (add-cells exp-cell inc-cell tra-cell))))
 
-          (add-report-line
-           (if standard-order? exp-table inc-table)
-           (string-append (G_ "Net income") period-for)
-           (string-append (G_ "Net loss") period-for)
-           net-income (* 2 (1- tree-depth)) exchange-fn #f #f)
+           ;; single-column
+           (standard-order?
+            (add-revenue-table build-table)
+            (add-rule build-table)
+            (unless (null? trading-accounts)
+              (add-trading-table build-table)
+              (add-rule build-table))
+            (add-expense-table build-table)
+            (add-rule build-table)
+            (add-report-line
+             build-table
+             (string-append (G_ "Net income") period-for)
+             (string-append (G_ "Net loss") period-for)
+             net-income (* 2 (1- tree-depth)) exchange-fn #f #f))
 
-          ;; add the sections in the desired order to document
-          (let ((build-table (gnc:make-html-table))
-                (inc-cell (gnc:make-html-table-cell inc-table))
-                (tra-cell (if (null? trading-accounts)
-                              (gnc:html-make-empty-cell)
-                              (gnc:make-html-table-cell tra-table)))
-                (exp-cell (gnc:make-html-table-cell exp-table)))
-            (define (add-cells . lst) (gnc:html-table-append-row! build-table lst))
-            (cond
-             ((and two-column? standard-order?)
-              (add-cells inc-cell tra-cell exp-cell))
+           (else
+            (add-expense-table build-table)
+            (add-rule build-table)
+            (unless (null? trading-accounts)
+              (add-trading-table build-table)
+              (add-rule build-table))
+            (add-revenue-table build-table)
+            (add-rule build-table)
+            (add-report-line
+             build-table
+             (string-append (G_ "Net income") period-for)
+             (string-append (G_ "Net loss") period-for)
+             net-income (* 2 (1- tree-depth)) exchange-fn #f #f)))
 
-             (two-column?
-              (add-cells exp-cell inc-cell tra-cell))
+          (gnc:html-table-set-style!
+           build-table "td"
+           'attribute '("align" "left")
+           'attribute '("valign" "top"))
 
-             (standard-order?
-              (add-cells inc-cell)
-              (unless (null? trading-accounts) (add-cells tra-cell))
-              (add-cells exp-cell))
-
-             (else
-              (add-cells exp-cell)
-              (add-cells inc-cell)
-              (unless (null? trading-accounts) (add-cells tra-cell))))
-
-            (gnc:html-table-set-style!
-             build-table "td"
-             'attribute '("align" "left")
-             'attribute '("valign" "top"))
-            (gnc:html-document-add-object! doc build-table))
+          (gnc:html-document-add-object! doc build-table)
 
           ;; add currency information if requested
           (gnc:report-percent-done 90)
           (when show-rates?
             (gnc:html-document-add-object!
-             doc (gnc:html-make-exchangerates
-                  report-commodity exchange-fn accounts)))
+             doc (gnc:html-make-rates-table
+                  report-commodity price-fn accounts)))
           (gnc:report-percent-done 100)))
 
     (gnc:report-finished)
@@ -583,7 +563,7 @@
   (income-statement-renderer-internal report-obj pnl-reportname))
 
 
-(gnc:define-report 
+(gnc:define-report
  'version 1
  'name is-reportname
  'report-guid "0b81a3bdfd504aff849ec2e8630524bc"
@@ -593,7 +573,7 @@
 
 ;; Also make a "Profit & Loss" report, even if it's the exact same one,
 ;; just relabeled.
-(gnc:define-report 
+(gnc:define-report
  'version 1
  'name pnl-reportname
  'report-guid "8758ba23984c40dea5527f5f0ca2779e"
